pub const KEYWORDS: [&str; 110] = [
    "BASE",
    "PREFIX",
    "SELECT",
    "DISTINCT",
    "REDUCED",
    "FROM",
    "NAMED",
    "WHERE",
    "UNION",
    "OPTIONAL",
    "MINUS",
    "GRAPH",
    "SERVICE",
    "SILENT",
    "FILTER",
    "BIND",
    "VALUES",
    "AS",
    "GROUP",
    "BY",
    "HAVING",
    "ORDER",
    "DESC",
    "ASC",
    "LIMIT",
    "OFFSET",
    "CONSTRUCT",
    "DESCRIBE",
    "ASK",
    "STR",
    "LANG",
    "STR",
    "LANG",
    "LANGMATCHES",
    "DATATYPE",
    "BOUND",
    "IRI",
    "URI",
    "BNODE",
    "RAND",
    "ABS",
    "CEIL",
    "FLOOR",
    "ROUND",
    "CONCAT",
    "STRLEN",
    "UCASE",
    "LCASE",
    "ENCODE_FOR_URI",
    "CONTAINS",
    "STRSTARTS",
    "STRENDS",
    "STRBEFORE",
    "STRAFTER",
    "YEAR",
    "MONTH",
    "DAY",
    "HOURS",
    "MINUTES",
    "SECONDS",
    "TIMEZONE",
    "TZ",
    "NOW",
    "UUID",
    "STRUUID",
    "MD5",
    "SHA1",
    "SHA256",
    "SHA384",
    "SHA512",
    "COALESCE",
    "IF",
    "STRLANG",
    "STRDT",
    "sameTerm",
    "isIRI",
    "isURI",
    "isBLANK",
    "isLITERAL",
    "isNUMERIC",
    "COUNT",
    "SUM",
    "MIN",
    "MAX",
    "AVG",
    "SAMPLE",
    "GROUP_CONCAT",
    "SEPARATOR",
    "SUBSTR",
    "REGEX",
    "EXISTS",
    "IN",
    "NOT",
    "LOAD",
    "INTO",
    "CLEAR",
    "ALL",
    "NAMED",
    "DEFAULT",
    "DROP",
    "ADD",
    "TO",
    "MOVE",
    "COPY",
    "INSERT",
    "DATA",
    "DELETE",
    "WITH",
    "USING",
    "CREATE",
];
